//+------------------------------------------------------------------+
//|                                                 Nina_Expert1.mq4 |
//+------------------------------------------------------------------+
#define MAGICMA  20060219

//----input parameters
extern double Lots               = 0.1;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 3;
extern int    Slippage           = 3;

extern double MovingPeriod       = 50;
extern int PeriodWATR=10;
extern double Kwatr=1.0000;
extern int HighLow=0;
extern int CalculatedBars = 512;
extern int Confirmed_Bars = 2;

extern double TrailingStop = 34;
extern double TakeProfit   = 34;

//---- indicator buffers
double LineMinBuffer[2000];
double LineMidBuffer[2000];

string   TradeComment = "Nina_ExpertSystem";

//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
}

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
  
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma0, ma1;
   int    i, res;
   bool   ema_downcross    =false;
   bool   ema_upcross      =false;
   bool   stepma_downcross =false;
   bool   stepma_upcross   =false;
   
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
   
   for(i=0; i<=Confirmed_Bars; i++) {
      ma0=iMA(NULL,0,MovingPeriod,0,MODE_EMA,PRICE_CLOSE,i);
      ma1=iMA(NULL,0,MovingPeriod,0,MODE_EMA,PRICE_CLOSE,i+1);
      if((Open[i]>ma0) && (Open[i+1]<ma1)) {
         ema_upcross = true;
         break;
      }
      if((Open[i]<ma0) && (Open[i+1]>ma1)) {
         ema_downcross = true;
         break;
      }
   }
   for(i=0; i<=Confirmed_Bars; i++) {
      if((LineMinBuffer[i+1]>LineMidBuffer[i+1]) && (LineMinBuffer[i+2]<LineMidBuffer[i+2])) {
         stepma_upcross = true;
         break;
      }
      if((LineMinBuffer[i+1]<LineMidBuffer[i+1]) && (LineMinBuffer[i+2]>LineMidBuffer[i+2])) {
         stepma_downcross = true;
         break;
      }
   }
///---- get Moving Average 
   ma0=iMA(NULL,0,MovingPeriod,0,MODE_EMA,PRICE_CLOSE,0);
//---- Buy conditions
   if((Open[0]>ma0) && (LineMinBuffer[1] > LineMidBuffer[1]) && ((Open[0]-ma0) < 20*Point())){
      if(ema_upcross==true && stepma_upcross==true) {
//      ema_upcross && stepma_upcross) {
   if(TakeProfit>5) 
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,Slippage,Ask-TrailingStop*Point(),Ask+TakeProfit*Point(),"",MAGICMA,0,Blue);
   else
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,Slippage,Ask-TrailingStop*Point(),0,"",MAGICMA,0,Blue);      
      return;
      }
   }
//---- sell conditions
   if(Open[0]<ma0 && LineMinBuffer[1] < LineMidBuffer[1] && ma0-Open[0] < 20*Point()) {
      if(ema_downcross==true && stepma_downcross==true) {
   if(TakeProfit>5)
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,Slippage,Bid+TrailingStop*Point(),Bid-TakeProfit*Point(),"",MAGICMA,0,Red);
   else
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,Slippage,Bid+TrailingStop*Point(),0,"",MAGICMA,0,Red);
   
      return;
     }
   }
//----
}

//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   int res;
   int trend2 = 0;

//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
//   if(totalBars == Bars) return;

   if(LineMinBuffer[1]>LineMinBuffer[2]) trend2=1;
   else if(LineMinBuffer[1]<LineMinBuffer[2]) trend2=-1;   

//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(trend2==-1) OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,White);
         continue;
        }
      if(OrderType()==OP_SELL)
        {
         if(trend2==1) OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,White);
         continue;
        }
     }
//----
  }
  
void CheckForStop()
{
   for(int i=0;i<OrdersTotal();i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
      {
         if((OrderStopLoss()<(Bid-Point*TrailingStop)) || (OrderStopLoss()==0))
         {
            OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green);
            continue;
         }
      }
      if(OrderType()==OP_SELL)
      {
         if((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
         {
            OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red);
            continue;
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int CalculateStepMA()
  {
   int      i,shift,TrendMin,TrendMax,TrendMid;
   double   SminMin0,SmaxMin0,SminMin1,SmaxMin1,SumRange,dK,WATR0,WATRmax,WATRmin,WATRmid;
   double   SminMax0,SmaxMax0,SminMax1,SmaxMax1,SminMid0,SmaxMid0,SminMid1,SmaxMid1;
   double   linemin,linemax,linemid,Stoch1,Stoch2,bsmin,bsmax;
   static bool turn_alarm = true;
   	
   int StepSizeMin,StepSizeMax,StepSizeMid;
   double min,max,mid,h,l,c;
  
   if(CalculatedBars>Bars || CalculatedBars == 0 ) CalculatedBars = Bars;
 
   for(shift=CalculatedBars-1;shift>=0;shift--){	
	  SumRange=0;
	  for (i=PeriodWATR-1;i>=0;i--){ 
         dK = 1+1.0*(PeriodWATR-i)/PeriodWATR;
         SumRange+= dK*MathAbs(High[i+shift]-Low[i+shift]);
         }
	  WATR0 = SumRange/PeriodWATR;
	
	  WATRmax=MathMax(WATR0,WATRmax);
	  if (shift==CalculatedBars-1-PeriodWATR) WATRmin=WATR0;
	  WATRmin=MathMin(WATR0,WATRmin);
	
	  StepSizeMin=MathRound(Kwatr*WATRmin/Point);
	  StepSizeMax=MathRound(Kwatr*WATRmax/Point);
	  StepSizeMid=MathRound(Kwatr*0.5*(WATRmax+WATRmin)/Point);
	  
     min = Kwatr*WATRmin;
     max = Kwatr*WATRmax;
     mid = Kwatr*0.5*(WATRmax+WATRmin);
	  
	  //b = iBarShift(Symbol(),tPeriod,Time[shift]);
	  c = Close[shift];//iClose(Symbol(),tPeriod,b);
	  h = High[shift];//iHigh(Symbol(),tPeriod,b)
	  l = Low[shift];//iLow(Symbol(),tPeriod,b)
	  
	  if (HighLow>0){
	    SmaxMin0=l+2*min;
	    SminMin0=h-2*min;
	  
	    SmaxMax0=l+2*max;
	    SminMax0=h-2*max;
	  
	    SmaxMid0=l+2*mid;
	    SminMid0=h-2*mid;
	  
	    if(c>SmaxMin1) TrendMin=1; 
	    if(c<SminMin1) TrendMin=-1;
	  
	    if(c>SmaxMax1) TrendMax=1; 
	    if(c<SminMax1) TrendMax=-1;
	  
	    if(c>SmaxMid1) TrendMid=1; 
	    if(c<SminMid1) TrendMid=-1;
	    }
	 
	  if (HighLow == 0){
	    SmaxMin0=c+2*min;
	    SminMin0=c-2*min;
	  
	    SmaxMax0=c+2*max;
	    SminMax0=c-2*max;
	  
	    SmaxMid0=c+2*mid;
	    SminMid0=c-2*mid;
	  
	    if(c>SmaxMin1) TrendMin=1; 
	    if(c<SminMin1) TrendMin=-1;
	  
	    if(c>SmaxMax1) TrendMax=1; 
	    if(c<SminMax1) TrendMax=-1;
	  
	    if(c>SmaxMid1) TrendMid=1; 
	    if(c<SminMid1) TrendMid=-1;
	    }
	 	
	  if(TrendMin>0 && SminMin0<SminMin1) SminMin0=SminMin1;
	  if(TrendMin<0 && SmaxMin0>SmaxMin1) SmaxMin0=SmaxMin1;
		
	  if(TrendMax>0 && SminMax0<SminMax1) SminMax0=SminMax1;
	  if(TrendMax<0 && SmaxMax0>SmaxMax1) SmaxMax0=SmaxMax1;
	  
	  if(TrendMid>0 && SminMid0<SminMid1) SminMid0=SminMid1;
	  if(TrendMid<0 && SmaxMid0>SmaxMid1) SmaxMid0=SmaxMid1;
	  
	  
	  if (TrendMin>0) linemin=SminMin0+min;
	  if (TrendMin<0) linemin=SmaxMin0-min;
	  
	  if (TrendMax>0) linemax=SminMax0+max;
	  if (TrendMax<0) linemax=SmaxMax0-max;
	  
	  if (TrendMid>0) linemid=SminMid0+mid;
	  if (TrendMid<0) linemid=SmaxMid0-mid;
	  
	  bsmin=linemax-max;
	  bsmax=linemax+max;
	  Stoch1=(linemin-bsmin)/(bsmax-bsmin);
	  Stoch2=(linemid-bsmin)/(bsmax-bsmin);
	  
	  LineMinBuffer[shift]=Stoch1;
	  LineMidBuffer[shift]=Stoch2;
	  
	  
	  SminMin1=SminMin0;
	  SmaxMin1=SmaxMin0;
	  
	  SminMax1=SminMax0;
	  SmaxMax1=SmaxMax0;
	  
	  SminMid1=SminMid0;
	  SmaxMid1=SmaxMid0;
	  
   }
	     
	return(0);	
 }
 
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
   CalculateStepMA();
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
   if(TrailingStop>0) CheckForStop();
//----
  }
//+------------------------------------------------------------------+


